@@ -1,6 +1,8 @@
 import os
 import sys
-from ConfigParser import SafeConfigParser
+import ConfigParser
+
+from cached_property import cached_property
 
 from .task import Task
 
@@ -8,7 +10,7 @@ class Config(object):
     def __init__(self, config_path=None, debug=False):
         self.home = os.path.abspath(os.path.dirname(config_path or '.'))
         self.debug = debug
-        self.config = SafeConfigParser()
+        self.config = ConfigParser.SafeConfigParser()
         with open(config_path) as f:
             self.config.readfp(f)
 
@@ -21,7 +23,7 @@ class Config(object):
 
     def load_task(self, name):
         task_path = self.config.get(name, 'task')
-        module = import_task(task_path)
+        module = import_string(task_path)
         return Task(name, module)
 
     def tasks(self):
@@ -31,11 +33,24 @@ class Config(object):
         return dict(self.config.items(name))
 
     def state_dir(self):
-        path = self.config.get('DEFAULT', 'state_dir') or './state'
+        path = self.config.get('DEFAULT', 'state_dir')
+
         return os.path.join(self.home, path)
 
+    @cached_property
+    def serializer(self):
+        try:
+            serializer_module = self.config.get('DEFAULT', 'serializer')
+        except ConfigParser.NoOptionError:
+            serializer_module = 'json'
+        module = import_string(serializer_module)
 
-def import_task(path):
+        assert hasattr(module, 'loads') and hasattr(module, 'dumps'), 'Serializer should have loads and dumps methods'
+
+        return module
+
+
+def import_string(path):
     if '.' not in path:
         return __import__(path)
     else:
