@@ -2,12 +2,12 @@ import time
 import logging
 import types
 import gevent
-import json
-import ujson
 from gevent.queue import Queue, Empty
 
+from cached_property import cached_property
+
 import kafka.common
-from kafka import SimpleConsumer, KeyedProducer
+from kafka import SimpleConsumer, KeyedProducer, protocol as kafka_protocol
 from kafka.conn import DEFAULT_SOCKET_TIMEOUT_SECONDS
 
 
@@ -57,13 +57,12 @@ class Task(object):
 
 
 class TaskInstance(object):
-
-    def __init__(self, kafka, task, partition_id, state_mgr, config):
+    def __init__(self, kafka, task, partition_id, state_mgr, serializer, config):
         self.kafka = kafka
         self.task = task
         self.partition_id = partition_id
-        self.producer = KeyedProducer(self.kafka)
         self.state_mgr = state_mgr
+        self.serializer = serializer
         self.last_window_time = time.time()
         self.logger = logging.getLogger('pymza.task.{0}'.format(self.name))
         self.queue = Queue(maxsize=1000)
@@ -93,6 +92,17 @@ class TaskInstance(object):
         else:
             self.window_time = None
 
+    @cached_property
+    def producer(self):
+        codec_name = self.config.get('producer_codec', 'SNAPPY')
+        codec = getattr(kafka_protocol, 'CODEC_{0}'.format(codec_name.upper()), None)
+        if codec is None:
+            raise ValueError('Unable to resolve codec {0}.'.format(codec_name))
+
+        return KeyedProducer(self.kafka, codec=codec)
+
+
+
     @property
     def name(self):
         return self.task.name
@@ -146,7 +156,13 @@ class TaskInstance(object):
                 self.maybe_commit()
                 continue
 
-            message = SourceMessage(json.loads(message_envelope.message.value), source=consumer.topic, key=message_envelope.message.key)
+            try:
+                message_data = self.serializer.loads(message_envelope.message.value)
+            except Exception:
+                self.logger.exception('Failed to deserialize message data {0!r}'.format(message_envelope.message.value))
+                raise
+
+            message = SourceMessage(message_data, source=consumer.topic, key=message_envelope.message.key)
             result = self.task.process(message, self.state)
             self.handle_task_result(result)
 
@@ -204,8 +220,8 @@ class TaskInstance(object):
                     raise
                 if key is None:
                     self.logger.error('Task yielded data with no key. Message not sent.')
-                try:                    
-                    self.producer.send(ensure_str(topic), ensure_str(key), ujson.dumps(value))
+                try:
+                    self.producer.send(ensure_str(topic), ensure_str(key), self.serializer.dumps(value))
                 except Exception:
                     self.logger.exception('Failed to send message to {0}:{1}'.format(topic, key))
                     raise
